
uniform vec4 color;
uniform vec3 light;

varying vec4 var_pos;
varying vec4 var_normal;

void main()
{


  //vec3 refl = reflect(-light, var_normal);
  //vec3 view = normalize(-vec3(var_pos)/var_pos.w);

  //float lamb = max(dot(light,-var_normal), 0.0);


  //gl_FragColor = color*lamb;



}


